// -*- C++ -*-
/*!
 * @file ExecutionContextBase.h
 * @brief ExecutionContext base class
 * @date $Date: 2008-01-14 07:48:55 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2007, 2011-2012
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: ExecutionContextBase.h 3097 2017-12-09 23:48:36Z n-ando $
 *
 */

#ifndef RTC_EXECUTIONCONTEXTBASE_H
#define RTC_EXECUTIONCONTEXTBASE_H

#include <coil/Factory.h>

#include <rtm/idl/RTCSkel.h>
#include <rtm/idl/OpenRTMSkel.h>
#include <rtm/Factory.h>
#include <rtm/ExecutionContextProfile.h>
#include <rtm/ExecutionContextWorker.h>

#ifdef WIN32
#pragma warning( disable : 4290 )
#endif

#define DEFAULT_EXECUTION_RATE 1000

namespace coil
{
  class Properties;
}

namespace RTC
{
  /*!
   * @if jp
   * @class ExecutionContextBase
   * @brief ExecutionContextpNX
   *
   * EC̎NXł́ÅNXpAECCORBAIy[
   * VȂ΂ȂȂBɁAۂɃWbN쓮邽
   * ߁A̖񑩂ɑExecutionContextBase̊֐ĂяoKv
   * BECCORBAIy[V͈ȉ̂̂Aꂼ
   * ExecutionContextBasẽo֐ɑΉĂB
   *
   * - is_running(): ExecutionContextBase::isRunning()
   * - start(): ExecutionContextBase::start()
   * - stop(): ExecutionContextBase::stop()
   *
   * - get_rate(): ExecutionContextBase::gatRate()
   * - set_rate(): ExecutioinContextBase::setRate()
   *
   * - add_component(): ExecutionContextBase::addComponent()
   * - remove_component(): ExecutionContextBase::removeComponent()
   *
   * - activate_component(): ExecutionContextBase::activateComponent()
   * - deactivate_component(): ExecutionContextBase::deactivateComponent()
   * - reset_component(): ExecutionContextBase::resetComponent()
   *
   * - get_component_state(): ExecutionContextBase::getComponentState()
   * - get_kind(): ExecutionContextBase::getKind()
   * - get_profile(): ExecutionContextBase::getProfile()
   *
   * @par sԂɊ֌W֐Ǝ@
   * - is_running(): ExecutionContextBase::isRunning()
   * - start(): ExecutionContextBase::start()
   * - stop(): ExecutionContextBase::stop()
   *
   * sԂɊ֌W֐́Ais_running(), start(), stop() 3
   * BExecutionContextBaseł͒P running/stopped ̃tO
   * ĂAstart/stopŃtOON/OFF؂ւAis_running()ŏԓǂ
   * osĂBʏAEC̎NXł́Aprotected ȉz
   * o֐ onStarting(), onStarted(), onStopping(), onStopped() ֐
   * ŁACORBAIy[Vȉ̂悤ɎKvB
   *
   * is_running() CORBAIy[Vł́AP
   * ExecutionContextBase  isRunning() ĂяołB̊֐
   * Ɋ֘A protected z֐onIsRunning() pӂĂ邪A
   * ʏɎKv͂ȂBāA݂ running/stopped Ԃ
   * ꍇɂ̊֐𗘗p邱Ƃł邪͂ȂB
   *
   * <pre>
   * public:
   *  CORBA::Boolean is_runing()
   *  {
   *    return ExecutionContextBase::isRunning();
   *  }
   * protected:
   *  CORBA::Boolean onIsRunning(CORBA::Boolean running)
   *  {
   *    return running;
   *  }
   * </pre>
   *
   * start(), stop() CORBAIy[Vł́Aʏ
   * ExecutionContextBase  start(), stop() ֐Ăяo悤B
   * ̊֐Ɋ֘A protected z֐́Astart()  stop() 
   * Ăꂼ2Â onStarting(), onStarted(), 
   * onStopping(), onStopped() ֐BEC̎NXɂẮA
   * ꂼȉ̂悤ɎB
   *
   * <pre>
   *  RTC::ReturnCode_t start()
   *  {
   *    return ExecutionContextBase::start();
   *  }
   *  RTC::ReturnCode_t stop()
   *  {
   *    return ExecutionContextBase::stop();
   *  }
   * protected:
   *  RTC::ReturnCode_t onStarting()
   *  {
   *    RTC::ReturnCode_t ret = // XbhJn鏈Ȃ
   *    return ret;
   *  }
   *  RTC::ReturnCode_t onStarted()
   *  {
   *    RTC::ReturnCode_t ret = // XbhJn鏈Ȃ
   *    return ret;
   *  }
   *  RTC::ReturnCode_t onStopping()
   *  {
   *    // Xbh~鏈Ȃ
   *    return retcode;
   *  }
   *  RTC::ReturnCode_t onStopped()
   *  {
   *    // Xbh~鏈Ȃ
   *    return retcode;
   *  }
   * </pre>
   *
   * @par sɊւ֐Ǝ@
   * - get_rate(): ExecutionContextBase::gatRate()
   * - set_rate(): ExecutioinContextBase::setRate()
   *
   * sɊւ֐ set_rate(), get_rate() 2ނB
   * sReLXg set_rate() ɂw肳𗘗p
   * ꍇAev[g֐ onSetRate() I[o[ChB
   * onSetRate() ͈ double ^̎A̒l͐Ȓlł
   * Ƃۏ؂ĂBonSetRate() RTC::RTC_OK ȊO̒lԂ
   * AECProfile̎͐ݒ肳ȑO̒lێ邱Ƃۏ؂
   * B
   *
   * set_rate() l get_rate() ĂяoonGetRate() Ăяo
   * A͒ʏI[o[ChKv͂ȂBAget_rate() 
   * ԂlύXꍇAonGetRate() I[o[Ch邱Ƃł
   * l邱ƂłBA͐ȂB
   *
   * <pre>
   * public:
   *  RTC::ReturnCode_t set_rate(double rate)
   *  {
   *    return setRate(rate);
   *  }
   *  double get_rate(void) const
   *  {
   *    return getRate();
   *  }
   * protected:
   *  virtual RTC::ReturnCode_t onSetRate(double rate)
   *  {
   *    RTC::ReturnCode_t ret = // ݒ肷鉽炩̏
   *    if (ret != RTC::RTC_OK)
   *      {
   *        RTC_ERROR(("Error message"));
   *      }
   *    return ret;
   *  }
   *  virtual double onGetRate(rate)
   *  {
   *    // get_rate() ԂlHꍇ
   *    // ʏ͂̊֐Kv͂ȂB
   *    return rate;
   *  }
   * </pre>
   *
   * @par R|[lg̒ǉƍ폜Ɋւ֐
   * - add_component(): ExecutionContextBase::addComponent()
   * - remove_component(): ExecutionContextBase::removeComponent()
   *
   * R|[lg̒ǉƍ폜Ɋւ֐́Aadd_component(),
   * remove_component() ̓ނBsReLXg̎NX
   * ẮAExecutionContextBase ̂ꂼ addComponent(),
   * removeComponent() Ăяo`ŎsB̊֐Ɋ֘A
   * protected z֐ onAddingComponent(), onAddedComponent(),
   * onRemovingComponent(), onRemovedComponent() 4ނBA
   * ̉z֐͒ʏI[o[ChKv͂ȂAgp͐
   * ȂB
   *
   * <pre>
   * public:
   *  RTC::ReturnCode_t add_component(RTC::LightweightRTObject_ptr comp)
   *  {
   *    return ExecutionContextBase::addComponent(comp);
   *  }
   *  RTC::ReturnCode_t remove_component(RTC::LightweightRTObject_ptr comp)
   *  {
   *    return ExecutionContextBase::removeComponent(comp);
   *  }
   * protected:
   *  virtual RTC::ReturnCode_t
   *  onAddingComponent(RTC::LightweightRTObject rtobj)
   *  {
   *     // R|[lgǉɎsLq
   *     // RTC::RTC_OK ȊOԂꍇAR|[lg̒ǉ͍sȂB
   *     return RTC::RTC_OK;
   *  }
   *  virtual RTC::ReturnCode_t
   *  onAddedComponent(RTC::LightweightRTObject rtobj)
   *  {
   *     // R|[lgǉɎsLq
   *     // RTC::RTC_OK ȊOԂꍇAremoveComponent() ĂяoA
   *     // ǉꂽR|[lg폜B
   *     return RTC::RTC_OK;
   *  }
   *  virtual RTC::ReturnCode_t
   *  onRemovingComponent(RTC::LightweightRTObject rtobj)
   *  {
   *     // R|[lg폜ɎsLq
   *     // RTC::RTC_OK ȊOԂꍇAR|[lg̍폜͍sȂB
   *     return RTC::RTC_OK;
   *  }
   *  virtual RTC::ReturnCode_t
   *  onRemovedComponent(RTC::LightweightRTObject rtobj)
   *  {
   *     // R|[lgǉɎsLq
   *     // RTC::RTC_OK ȊOԂꍇAaddComponent() ĂяoA
   *     // 폜ꂽR|[lgĂђǉB
   *     return RTC::RTC_OK;
   *  }
   * </pre>
   *
   * @par R|[lg̃ANeBuɊւ֐
   * - activate_component(): ExecutionContextBase::activateComponent()
   * - deactivate_component(): ExecutionContextBase::deactivateComponent()
   * - reset_component(): ExecutionContextBase::resetComponent()
   *
   * R|[lg̃ANeBuɊւ֐́A
   * activate_component(), deactivate_component(), reset_component() 
   * OނBsReLXg̎NXɂẮA
   * ExecutionContextBase ̂ꂼ activateComponent(),
   * deactivateComponent(), resetComponent() Ăяo`ŎsB
   * ̊֐Ɋ֘A protected z֐
   * onActivatingComponent(), onAtivatingComponent(),
   * onActivatedComponent(), onDeactivatingComponent(),
   * onDeactivatedComponent(), onResettingComponent(),
   * onResetComponent() 6ނBẢz֐͒ʏI[
   * o[ChKv͂ȂAgp͐ȂB
   *
   * <pre>
   * public:
   *  RTC::ReturnCode_t add_component(RTC::LightweightRTObject_ptr comp)
   *  {
   *    return ExecutionContextBase::addComponent(comp);
   *  }
   *  RTC::ReturnCode_t remove_component(RTC::LightweightRTObject_ptr comp)
   *  {
   *    return ExecutionContextBase::removeComponent(comp);
   *  }
   * protected:
   *  virtual RTC::ReturnCode_t
   *  onAddingComponent(RTC::LightweightRTObject rtobj)
   *  {
   *    // R|[lgǉɎsLq
   *    // RTC::RTC_OK ȊOԂꍇAR|[lg̒ǉ͍sȂB
   *    return RTC::RTC_OK;
   *  }
   *  virtual RTC::ReturnCode_t
   *  onAddedComponent(RTC::LightweightRTObject rtobj)
   *  {
   *    // R|[lgǉɎsLq
   *    // RTC::RTC_OK ȊOԂꍇAremoveComponent() ĂяoA
   *    // ǉꂽR|[lg폜B
   *    return RTC::RTC_OK;
   *  }
   *  virtual RTC::ReturnCode_t
   *  onRemovingComponent(RTC::LightweightRTObject rtobj)
   *  {
   *    // R|[lg폜ɎsLq
   *    // RTC::RTC_OK ȊOԂꍇAR|[lg̍폜͍sȂB
   *    return RTC::RTC_OK;
   *  }
   *  virtual RTC::ReturnCode_t
   *  onRemovedComponent(RTC::LightweightRTObject rtobj)
   *  {
   *    // R|[lgǉɎsLq
   *    // RTC::RTC_OK ȊOԂꍇAaddComponent() ĂяoA
   *    // 폜ꂽR|[lgĂђǉB
   *    return RTC::RTC_OK;
   *  }
   * </pre>
   *
   * @par sReLXg̏擾Ɋւ֐
   * - get_component_state(): ExecutionContextBase::getComponentState()
   * - get_kind(): ExecutionContextBase::getKind()
   * - get_profile(): ExecutionContextBase::getProfile()
   *
   * sReLXg̏擾Ɋւ֐́Aget_component_state(),
   * get_kind(), get_profile() 3ނBsReLXg̎N
   * XɂẮAExecutionContextBase ̂ꂼ
   * getComponentState(), getKind(), getProfile() Ăяo`Ŏ
   * sB̊֐Ɋ֘A protected z֐
   * onGetComponentState(), onGetKind(), onGetProfile() 3ނB
   * ̉z֐͒ʏI[o[ChKv͂ȂAgp͐
   * BAԂύXꍇ́Å֐K؂Ɏ
   * 邱ƂŁAĂяoɕԂl㏑邱ƂłB
   *
   * <pre>
   * public:
   *  LifeCycleState get_component_state(RTC::LightweightRTObject_ptr comp)
   *  {
   *    return getComponentState(comp);
   *  }
   *  ExecutionKind PeriodicExecutionContext::get_kind()
   *  {
   *    return getKind();
   *  }
   *  ExecutionContextProfile* get_profile()
   *  {
   *    return getProfile();
   *  }
   *
   * protected:
   *  virtual LifeCycleState onGetComponentState(LifeCycleState state)
   *  { // Ԃstateꍇ͂̊֐
   *    return state;
   *  }
   *  virtual ExecutionKind onGetKind(ExecutionKind kind)
   *  { // Ԃkindꍇ͂̊֐
   *    return kind;
   *  }
   *  virtual ExecutionContextProfile*
   *  onGetProfile(ExecutionContextProfile*& profile)
   *  { // Ԃprofileꍇ͂̊֐
   *    return profile;
   *  }
   * </pre>
   *
   * [ExecutionContextBase]
   * <>
   *  |
   *  +--(1 1) [ExecutionContextWorker ] : EC  worker
   *  |         <>
   *  |          +--(1 0..) [RTObjectStateMachine] : StateMachine
   *  |
   *  +--(1 1) [ExecutionContextProfile] : EC Profile̊Ǘ
   * 
   * @since 0.4.0
   *
   * @else
   * @class ExecutionContextBase
   * @brief A base class for ExecutionContext
   *
   * A base class of ExecutionContext.
   *
   * @since 0.4.0
   *
   * @endif
   */
  class ExecutionContextBase
  {
  public:
    /*!
     * @if jp
     * @brief zfXgN^
     *
     * zfXgN^
     *
     * @else
     * @brief Virtual Destructor
     *
     * Virtual Destructor
     *
     * @endif
     */
    ExecutionContextBase(const char* name);

    /*!
     * @if jp
     * @brief zfXgN^
     *
     * zfXgN^
     *
     * @else
     * @brief Virtual Destructor
     *
     * Virtual Destructor
     *
     * @endif
     */
    virtual ~ExecutionContextBase(void);

    /*!
     * @if jp
     * @brief ExecutionContext̏s
     *
     * ExecutionContext̏
     *
     * @else
     * @brief Initialize the ExecutionContext
     *
     * This operation initialize the ExecutionContext
     *
     * @endif
     */
    virtual void init(coil::Properties& props);

    /*!
     * @if jp
     * @brief R|[lgoChB
     *
     * R|[lgoChB
     *
     * @else
     * @brief Bind the component.
     *
     * Bind the component.
     *
     * @endif
     */
    virtual RTC::ReturnCode_t bindComponent(RTC::RTObject_impl* rtc);

    //============================================================
    // implementation functions for EC's CORBA operation
    //============================================================
    /*!
     * @if jp
     * @brief ExecutionContext sԊmF֐
     *
     * ̑ ExecutionContext  Runnning Ԃ̏ꍇ true ԂB
     * ԂĺAstart() ֐Ă΂ꂽƂRunningԂƂȂ
     * true Astop() ֐Ă΂ꂽƂStoppedԂƂȂfalseԂB
     *
     * @return  (Running:trueAStopped: false)
     *
     * @else
     *
     * @brief Check for ExecutionContext running state
     *
     * This operation shall return true if the context is in the
     * Running state.  After calling the start() of this class, this
     * function returns true, after calling to stop function of it,
     * this function returns false.
     *
     * @return Working status (Running:trueAStopped:false)
     *
     * @endif
     */
    CORBA::Boolean isRunning();

    /*!
     * @if jp
     * @brief ExecutionContext ̎sJn
     *
     * ExecutionContext ̎sԂ Runnning Ƃ邽߂̃NGXg
     * sBExecutionContext ̏ԂJڂ
     * ComponentAction::on_startup ĂяoBQĂRTR|[
     * lgA܂ ExecutionContext Jn邱Ƃ͂ł
     * ȂBExecutionContext ͕Jn/~JԂƂłB
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief Start the ExecutionContext
     *
     * Request that the context enter the Running state.  Once the
     * state transition occurs, the ComponentAction::on_startup
     * operation will be invoked.  An execution context may not be
     * started until the RT-Components that participate in it have
     * been initialized.  An execution context may be started and
     * stopped multiple times.
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    RTC::ReturnCode_t start(void);

    /*!
     * @if jp
     * @brief ExecutionContext ̎s~
     *
     * ExecutionContext ̏Ԃ Stopped Ƃ邽߂̃NGXg𔭍s
     * BJڂꍇ́AComponentAction::on_shutdown Ăяo
     * BQĂRTR|[lgIO
     * ExecutionContext ~KvBExecutionContext ͕
     * Jn/~JԂƂłB
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief Stop the ExecutionContext
     *
     * Request that the context enter the Stopped state.  Once the
     * transition occurs, the ComponentAction::on_shutdown operation
     * will be invoked.  An execution context must be stopped before
     * the RT components that participate in it are finalized.  An
     * execution context may be started and stopped multiple times.
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual RTC::ReturnCode_t stop(void);

    /*!
     * @if jp
     * @brief ExecutionContext ̎s(Hz)擾
     *
     * Active ԂɂRTR|[lgs(P:Hz)擾
     * B
     *
     * @return (P:Hz)
     *
     * @else
     *
     * @brief Get execution rate(Hz) of ExecutionContext
     *
     * This operation shall return the rate (in hertz) at which its
     * Active participating RTCs are being invoked.
     *
     * @return Execution cycle(Unit:Hz)
     *
     * @endif
     */
    double getRate(void) const;
    coil::TimeValue getPeriod(void) const;
 
    /*!
     * @if jp
     * @brief ExecutionContext ̎s(Hz)ݒ肷
     *
     * Active ԂɂRTR|[lgs(P:Hz)ݒ肷
     * Bs̕ύX́ADataFlowComponentAction 
     * on_rate_changed ɂĊeRTR|[lgɓ`BB
     *
     * @param rate (P:Hz)
     *
     * @return ReturnCode_t ^̃^[R[h
     *         RTC_OK: I
     *         BAD_PARAMETER: ݒl̒l
     *
     * @else
     *
     * @brief Set execution rate(Hz) of ExecutionContext
     *
     * This operation shall set the rate (in hertz) at which this
     * contextfs Active participating RTCs are being called.  If the
     * execution kind of the context is PERIODIC, a rate change shall
     * result in the invocation of on_rate_changed on any RTCs
     * realizing DataFlowComponentAction that are registered with any
     * RTCs participating in the context.
     *
     * @param rate Execution cycle(Unit:Hz)
     *
     * @return The return code of ReturnCode_t type
     *         RTC_OK: Succeed
     *         BAD_PARAMETER: Invalid value. The value might be negative.
     *
     * @endif
     */
    RTC::ReturnCode_t setRate(double rate);

    /*!
     * @if jp
     * @brief RTR|[lgǉ
     *
     * w肵RTR|[lgQ҃XgɒǉBǉꂽRTR
     * |[lg attach_context Ă΂AInactive ԂɑJڂB
     * w肳ꂽRTR|[lgnull̏ꍇ́ABAD_PARAMETER Ԃ
     * Bw肳ꂽRTR|[lg DataFlowComponent ȊȌꍇ́A
     * BAD_PARAMETER ԂB
     *
     * @param comp ǉΏRTR|[lg
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief Add an RT-component
     *
     * The operation causes the given RTC to begin participating in
     * the execution context.  The newly added RTC will receive a call
     * to LightweightRTComponent::attach_context and then enter the
     * Inactive state.  BAD_PARAMETER will be invoked, if the given
     * RT-Component is null or if the given RT-Component is other than
     * DataFlowComponent.
     *
     * @param comp The target RT-Component for add
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    RTC::ReturnCode_t addComponent(RTC::LightweightRTObject_ptr comp);

    /*!
     * @if jp
     * @brief RTR|[lgQ҃Xg폜
     *
     * w肵RTR|[lgQ҃Xg폜B폜ꂽ
     * RTR|[lg detach_context Ă΂Bw肳ꂽRTR|[
     * lgQ҃Xgɓo^ĂȂꍇ́ABAD_PARAMETER 
     * B
     *
     * @param comp 폜ΏRTR|[lg
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief Remove the RT-Component from participant list
     *
     * This operation causes a participant RTC to stop participating in the
     * execution context.
     * The removed RTC will receive a call to
     * LightweightRTComponent::detach_context.
     * BAD_PARAMETER will be returned, if the given RT-Component is not
     * participating in the participant list.
     *
     * @param comp The target RT-Component for delete
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    RTC::ReturnCode_t removeComponent(RTC::LightweightRTObject_ptr comp);

    /*!
     * @if jp
     * @brief RTR|[lgANeBu
     *
     * Inactive ԂɂRTR|[lgActive ɑJڂAANeB
     * uB̑삪Ă΂ꂽʁAon_activate ĂяoBw
     * 肵RTR|[lgQ҃XgɊ܂܂Ȃꍇ́A
     * BAD_PARAMETER ԂBw肵RTR|[lg̏Ԃ
     * Inactive ȊȌꍇ́APRECONDITION_NOT_MET ԂB
     *
     * @param comp ANeBuΏRTR|[lg
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief Activate an RT-component
     *
     * The given participant RTC is Inactive and is therefore not
     * being invoked according to the execution contextfs execution
     * kind. This operation shall cause the RTC to transition to the
     * Active state such that it may subsequently be invoked in this
     * execution context.  The callback on_activate shall be called as
     * a result of calling this operation. This operation shall not
     * return until the callback has returned, and shall result in an
     * error if the callback does.
     *
     * @param comp The target RT-Component for activation
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    RTC::ReturnCode_t activateComponent(RTC::LightweightRTObject_ptr comp);

    /*!
     * @if jp
     * @brief RTR|[lgANeBu
     *
     * Inactive ԂɂRTR|[lgANeBuAInactive
     * ɑJڂB̑삪Ă΂ꂽʁAon_deactivate Ăяo
     * Bw肵RTR|[lgQ҃XgɊ܂܂Ȃꍇ́A
     * BAD_PARAMETER ԂBw肵RTR|[lg̏Ԃ
     * Active ȊȌꍇ́APRECONDITION_NOT_MET ԂB
     *
     *
     *


     * |


     
     * @param comp ANeBuΏRTR|[lg
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief Deactivate an RT-component
     *
     * The given RTC is Active in the execution context. Cause it to
     * transition to the Inactive state such that it will not be
     * subsequently invoked from the context unless and until it is
     * activated again.  The callback on_deactivate shall be called as
     * a result of calling this operation. This operation shall not
     * return until the callback has returned, and shall result in an
     * error if the callback does.
     *
     * @param comp The target RT-Component for deactivate
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    RTC::ReturnCode_t deactivateComponent(RTC::LightweightRTObject_ptr comp);

    /*!
     * @if jp
     * @brief RTR|[lgZbg
     *
     * Error ԂRTR|[lg̕A݂B̑삪Ă΂ꂽ
     * ʁAon_reset ĂяoBw肵RTR|[lgQ҃
     * XgɊ܂܂Ȃꍇ́ABAD_PARAMETER ԂBw肵RTR
     * |[lg̏Ԃ Error ȊȌꍇ́APRECONDITION_NOT_MET 
     * B
     *
     * @param comp ZbgΏRTR|[lg
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief Reset the RT-component
     *
     * Attempt to recover the RTC when it is in Error.  The
     * ComponentAction::on_reset callback shall be invoked. This
     * operation shall not return until the callback has returned, and
     * shall result in an error if the callback does. If possible, the
     * RTC developer should implement that callback such that the RTC
     * may be returned to a valid state.
     *
     * @param comp The target RT-Component for reset
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    RTC::ReturnCode_t resetComponent(RTC::LightweightRTObject_ptr comp);

    /*!
     * @if jp
     * @brief RTR|[lg̏Ԃ擾
     *
     * w肵RTR|[lg̏(LifeCycleState)擾Bw肵
     * RTR|[lgQ҃XgɊ܂܂Ȃꍇ́A
     * UNKNOWN_STATE ԂB
     *
     * @param comp Ԏ擾ΏRTR|[lg
     *
     * @return ݂̏(LifeCycleState)
     *
     * @else
     *
     * @brief Get RT-component's state
     *
     * This operation shall report the LifeCycleState of the given
     * participant RTC.  UNKNOWN_STATE will be returned, if the given
     * RT-Component is not inclued in the participant list.
     *
     * @param comp The target RT-Component to get the state
     *
     * @return The current state of the target RT-Component(LifeCycleState)
     *
     * @endif
     */
    RTC::LifeCycleState getComponentState(RTC::LightweightRTObject_ptr comp);
    const char* getStateString(RTC::LifeCycleState state);
    
    /*!
     * @if jp
     * @brief ExecutionKind 擾
     *
     * { ExecutionContext  ExecutionKind 擾
     *
     * @return ExecutionKind
     *
     * @else
     *
     * @brief Get the ExecutionKind
     *
     * This operation shall report the execution kind of the execution
     * context.
     *
     * @return ExecutionKind
     *
     * @endif
     */
    RTC::ExecutionKind getKind(void) const;

    /*!
     * @if jp
     * @brief Profile擾
     *
     * RTC::ExecutionContextProfile 擾B擾
     * ExecutionContextProfile ̏L͌ĂяoɂB擾ꂽI
     * uWFNgsvɂȂꍇAĂяoJӔC𕉂B
     *
     * @return RTC::ExecutionContextProfile
     *
     * @else
     * @brief Getting Profile
     *
     * This function gets RTC::ExecutionContextProfile.  The ownership
     * of the obtained ExecutionContextProfile is given to caller. The
     * caller should release obtained object when it is unneccessary
     * anymore.
     *
     * @return RTC::ExecutionContextProfile
     *
     * @endif
     */
    RTC::ExecutionContextProfile* getProfile(void);

    //============================================================
    // Delegated functions to ExecutionContextProfile
    //============================================================
    /*!
     * @if jp
     * @brief CORBA IuWFNgQƂ̎擾
     *
     * {IuWFNg ExecutioncontextService ƂĂ CORBA IuWF
     * NgQƂ擾B
     *
     * @return CORBA IuWFNgQ
     *
     * @else
     * @brief Get the reference to the CORBA object
     *
     * Get the reference to the CORBA object as
     * ExecutioncontextService of this object.
     *
     * @return The reference to CORBA object
     *
     * @endif
     */
    void setObjRef(RTC::ExecutionContextService_ptr ec_ptr);

    /*!
     * @if jp
     * @brief CORBA IuWFNgQƂ̎擾
     *
     * {IuWFNg ExecutioncontextService ƂĂ CORBA IuWF
     * NgQƂ擾B
     *
     * @return CORBA IuWFNgQ
     *
     * @else
     *
     * Get the reference to the CORBA object as
     * ExecutioncontextService of this object.
     *
     * @return The reference to CORBA object
     *
     * @endif
     */
    RTC::ExecutionContextService_ptr getObjRef(void) const;

    /*!
     * @if jp
     * @brief ExecutionKind 𕶎񉻂
     *
     * RTC::ExecutionKind Œ`Ă PERIODIC, EVENT_DRIVEN,
     * OTHER 𕶎񉻂B
     *
     * @param kind ExecutionKind
     * @return 񉻂ꂽExecutionKind
     *
     * @else
     *
     * @brief Converting ExecutionKind enum to string 
     *
     * This function converts enumeration (PERIODIC, EVENT_DRIVEN,
     * OTHER) defined in RTC::ExecutionKind to string.
     *
     * @param kind ExecutionKind
     * @return String of ExecutionKind
     *
     * @endif
     */
    const char* getKindString(RTC::ExecutionKind kind) const;

    /*!
     * @if jp
     * @brief ExecutionKind ݒ肷
     *
     *  ExecutionContext  ExecutionKind ݒ肷
     *
     * @param kind ExecutionKind
     *
     * @else
     *
     * @brief Set the ExecutionKind
     *
     * This operation sets the kind of the execution context.
     *
     * @param kind ExecutionKind
     *
     * @endif
     */
    RTC::ReturnCode_t setKind(RTC::ExecutionKind kind);

    /*!
     * @if jp
     * @brief OwnerR|[lgZbgB
     *
     * ECOwnerƂȂRTCZbgB
     *
     * @param comp OwnerƂȂRTR|[lg
     * @return ReturnCode_t ^̃^[R[h
     * @else
     * @brief Setting owner component of the execution context
     *
     * This function sets an RT-Component to be owner of the execution context.
     *
     * @param comp an owner RT-Component of this execution context
     * @return The return code of ReturnCode_t type
     * @endif
     */
    RTC::ReturnCode_t setOwner(RTC::LightweightRTObject_ptr comp);

    /*!
     * @if jp
     * @brief OwnerR|[lg̎QƂ擾
     *
     * ECOwnerłRTC̎QƂ擾B
     *
     * @return OwnerRTR|[lg̎Q
     * @else
     * @brief Getting a reference of the owner component
     *
     * This function returns a reference of the owner RT-Component of
     * this execution context
     *
     * @return a reference of the owner RT-Component
     * @endif
     */
    const RTC::RTObject_ptr getOwner() const;

    /*!
     * @if jp
     * @brief RTR|[lg̎Q҃Xg擾
     *
     * ݓo^ĂQRTC̃Xg擾B
     *
     * @return QRTC̃Xg
     *
     * @else
     *
     * @brief Getting participant RTC list
     *
     * This function returns a list of participant RTC of the execution context.
     *
     * @return Participants RTC list
     *
     * @endif
     */
    const RTC::RTCList& getComponentList() const;

    /*!
     * @if jp
     * @brief PropertiesZbg
     *
     * ExecutionContextProfile::properties ZbgB
     *
     * @param props ExecutionContextProfile::properties ɃZbgv
     *              peB[
     *
     * @else
     * @brief Setting Properties
     *
     * This function sets ExecutionContextProfile::properties by
     * coil::Properties.
     *
     * @param props Properties to be set to
     *              ExecutionContextProfile::properties.
     *
     * @endif
     */
    void setProperties(coil::Properties& props);

    /*!
     * @if jp
     * @brief Properties擾
     *
     * ExecutionContextProfile::properties 擾B
     *
     * @return coil::Propertiesɕϊꂽ
     *              ExecutionContextProfile::properties
     *
     * @else
     * @brief Setting Properties
     *
     * This function sets ExecutionContextProfile::properties by
     * coil::Properties.
     *
     * @param props Properties to be set to ExecutionContextProfile::properties.
     *
     * @endif
     */
    const coil::Properties getProperties() const;

    /*!
     * @if jp
     * @brief Profile擾
     *
     * RTC::ExecutionContextProfile 擾B
     *
     * @return RTC::ExecutionContextProfile
     *
     * @else
     * @brief Getting Profile
     *
     * This function gets RTC::ExecutionContextProfile.
     *
     * @return RTC::ExecutionContextProfile
     *
     * @endif
     */
    const RTC::ExecutionContextProfile& getProfile(void) const;
    // end of delegated functions to ExecutionContextProfile
    //============================================================

    //============================================================
    // Delegated functions to ExecutionContextWorker
    //============================================================
    bool isAllCurrentState(RTC::LifeCycleState state)
    {
      return m_worker.isAllCurrentState(state);
    };
    bool isAllNextState(RTC::LifeCycleState state)
    {
      return m_worker.isAllNextState(state);
    };
    bool isOneOfCurrentState(RTC::LifeCycleState state)
    {
      return m_worker.isOneOfCurrentState(state);
    };
    bool isOneOfNextState(RTC::LifeCycleState state)
    {
      return m_worker.isOneOfNextState(state);
    };
    
    void invokeWorker()       { m_worker.invokeWorker(); }
    void invokeWorkerPreDo()  { m_worker.invokeWorkerPreDo(); }
    void invokeWorkerDo()     { m_worker.invokeWorkerDo(); }
    void invokeWorkerPostDo() { m_worker.invokeWorkerPostDo(); }


  protected:
    // template virtual functions related to start/stop
    virtual bool onIsRunning(bool running) { return running; }
    virtual RTC::ReturnCode_t onStarting() { return RTC::RTC_OK; }
    virtual RTC::ReturnCode_t onStarted() { return RTC::RTC_OK; }
    virtual RTC::ReturnCode_t onStopping() { return RTC::RTC_OK; }
    virtual RTC::ReturnCode_t onStopped() { return RTC::RTC_OK; }

    // template virtual functions getting/setting execution rate
    virtual double onGetRate(double rate) const { return rate; }
    virtual double onSettingRate(double rate) { return rate; }
    virtual RTC::ReturnCode_t onSetRate(double rate) { return RTC::RTC_OK; }

    // template virtual functions adding/removing component
    virtual RTC::ReturnCode_t
    onAddingComponent(RTC::LightweightRTObject_ptr rtobj)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t
    onAddedComponent(RTC::LightweightRTObject_ptr rtobj)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t
    onRemovingComponent(RTC::LightweightRTObject_ptr rtobj)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t
    onRemovedComponent(RTC::LightweightRTObject_ptr rtobj)
    {
      return RTC::RTC_OK;
    }

    // template virtual functions related to activation/deactivation/reset
    virtual RTC::ReturnCode_t
    onActivating(RTC::LightweightRTObject_ptr comp)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t
    onWaitingActivated(RTC_impl::RTObjectStateMachine* comp, long int count)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t
    onActivated(RTC_impl::RTObjectStateMachine* comp,
                                     long int count)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t
    onDeactivating(RTC::LightweightRTObject_ptr comp)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t
    onWaitingDeactivated(RTC_impl::RTObjectStateMachine* comp, long int count)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t
    onDeactivated(RTC_impl::RTObjectStateMachine* comp, long int count)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t onResetting(RTC::LightweightRTObject_ptr comp)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t
    onWaitingReset(RTC_impl::RTObjectStateMachine* comp, long int count)
    {
      return RTC::RTC_OK;
    }
    virtual RTC::ReturnCode_t
    onReset(RTC_impl::RTObjectStateMachine* comp, long int count)
    {
      return RTC::RTC_OK;
    }

    virtual RTC::LifeCycleState
    onGetComponentState(RTC::LifeCycleState state)
    {
      return state;
    }
    virtual RTC::ExecutionKind
    onGetKind(RTC::ExecutionKind kind) const
    {
      return kind;
    }
    virtual RTC::ExecutionContextProfile*
    onGetProfile(RTC::ExecutionContextProfile*& profile)
    {
      return profile;
    }

  private:
    /*!
     * @if jp
     * @brief PropertiessReLXgZbg
     * @else
     * @brief Setting execution rate from given properties.
     * @endif
     */
    bool setExecutionRate(coil::Properties& props);
    /*!
     * @if jp
     * @brief PropertiesԑJڃ[hZbg
     * @else
     * @brief Setting state transition mode from given properties.
     * @endif
     */
    bool setTransitionMode(coil::Properties& props,
                           const char* key, bool& flag);
    /*!
     * @if jp
     * @brief PropertiesԑJTimeoutZbg
     * @else
     * @brief Setting state transition timeout from given properties.
     * @endif
     */
    bool setTimeout(coil::Properties& props, const char* key,
                    coil::TimeValue& timevalue);

    RTC::ReturnCode_t waitForActivated(RTC_impl::RTObjectStateMachine* rtobj);
    RTC::ReturnCode_t waitForDeactivated(RTC_impl::RTObjectStateMachine* rtobj);
    RTC::ReturnCode_t waitForReset(RTC_impl::RTObjectStateMachine* rtobj);
    
  protected:
    mutable RTC::Logger rtclog;

    RTC_impl::ExecutionContextWorker m_worker;
    RTC_impl::ExecutionContextProfile m_profile;

    coil::TimeValue m_activationTimeout;
    coil::TimeValue m_deactivationTimeout;
    coil::TimeValue m_resetTimeout;

    bool m_syncActivation;
    bool m_syncDeactivation;
    bool m_syncReset;
  };  // class ExecutionContextBase

  typedef coil::GlobalFactory<ExecutionContextBase> ExecutionContextFactory;

#if defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__)
  EXTERN template class DLL_PLUGIN coil::GlobalFactory<ExecutionContextBase>;
#endif
};  // namespace RTC

#ifdef WIN32
#pragma warning( default : 4290 )
#endif

#endif // RTC_EXECUTIONCONTEXTBASE_H
