// -*- C++ -*-
/*!
 * @file InPortPushConnector.h
 * @brief Push type connector class
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2009
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */

#ifndef RTC_INPORTPUSHCONNECTOR_H
#define RTC_INPORTPUSHCONNECTOR_H

#include <rtm/InPortConnector.h>
#include <rtm/InPortConsumer.h>
#include <rtm/PublisherBase.h>

namespace RTC
{
  class InPortProvider;

  /*!
   * @if jp
   * @class InPortPushConnector
   * @brief InPortPushConnector NX
   *
   * InPort  push ^f[^t[̂߂ Connector NXB̃Iu
   * WFNǵAڑ dataflow_type  push w肳ꂽꍇA
   * InPort ɂĐELAOutPortPushConnector Ƒ΂ɂȂāA
   * f[^|[g push ^̃f[^t[B̐ڑɑ΂āA
   * ̃f[^Xg[񋟂B Connector ΉB
   * Connector  ڑɐ UUID ` ID ɂʂB
   *
   * InPortPushConnector ͈ȉ̎O̃IuWFNgLǗB
   *
   * - InPortProvider
   * - Buffer
   *
   * OutPort ɏ܂ꂽf[^́AOutPortConnector ɂ
   * InPortProvider::put() Ƀf[^nB܂ꂽf[^
   * Connector  Buffer Ƀf[^܂B
   *
   * @since 1.0.0
   *
   * @else
   * @class InPortPushConnector
   * @brief InPortPushConnector class
   *
   * Connector class of InPort for push type dataflow.  When "push"
   * is specified as dataflow_type at the time of establishing
   * connection, this object is generated and owned by the InPort.
   * This connector and OutPortPushConnector make a pair and realize
   * push type dataflow of data ports.  One connector corresponds to
   * one connection which provides a data stream.  Connector is
   * distinguished by ID of the UUID that is generated at establishing
   * connection.
   *
   * InPortPushConnector owns and manages the following objects.
   *
   * - OutPortConsumer
   * - Buffer
   *
   * Data written into the OutPort are passed to the
   * InPortProvider::put() by OutPortConnector.  The data is written
   * into the buffer in the connector.
   *
   * @since 1.0.0
   *
   * @endif
   */
  class InPortPushConnector
    : public InPortConnector
  {
  public:
    DATAPORTSTATUS_ENUM

    /*!
     * @if jp
     * @brief RXgN^
     *
     * InPortPushConnector ̃RXgN^̓IuWFNgɉL
     * ɂƂBConnectorInfo ͐ڑ܂݁Ȁɏ]obt@
     * 𐶐BInPort C^[tF[X̃voC_IuWFNg
     * ̃|C^AL̂ŁAInPortPushConnector 
     * InPortProvider ̉̐ӔCBeCxgɑ΂R[ob
     * N@\񋟂 ConnectorListeners AK؂ȃ^C~OŃR[
     * obNĂяoBf[^obt@ InPortBase 񋟂
     * ꍇ͂̃|C^B
     *
     * @param info ConnectorInfo
     * @param provider InPortProvider
     * @param listeners ConnectorListeners ^̃XiIuWFNgXg
     * @param buffer CdrBufferBase ^̃obt@
     *
     * @elsek
     * @brief Constructor
     *
     * InPortPushConnector's constructor is given the following
     * arguments.  According to ConnectorInfo which includes
     * connection information, a buffer is created.
     * It is also given a pointer to the provider object for the
     * InPort interface.  The owner-ship of the pointer is owned by
     * this InPortPushConnector, it has responsibility to destruct
     * the InPortProvider.  InPortPushConnector also has
     * ConnectorListeners to provide event callback mechanisms, and
     * they would be called at the proper timing.  If data buffer is
     * given by InPortBase, the pointer to the buffer is also given
     * as arguments.
     *
     * @param info ConnectorInfo
     * @param provider InPortProvider
     * @param listeners ConnectorListeners type lsitener object list
     * @param buffer CdrBufferBase type buffer
     *
     * @endif
     */
    InPortPushConnector(ConnectorInfo info,
                        InPortProvider* provider,
                        ConnectorListeners& listeners,
                        CdrBufferBase* buffer = 0);

    /*!
     * @if jp
     * @brief fXgN^
     *
     * disconnect() Ă΂Aconsumer, publisher, buffer ́E폜B
     *
     * @else
     *
     * @brief Destructor
     *
     * This operation calls disconnect(), which destructs and deletes
     * the consumer, the publisher and the buffer.
     *
     * @endif
     */
    virtual ~InPortPushConnector();

    /*!
     * @if jp
     * @brief f[^̓ǂݏo
     *
     * obt@f[^ǂݏoBɓǂݏoꍇA߂l
     * PORT_OK ƂȂAdata ɓǂݏoꂽf[^i[BȊO
     * ̏ꍇɂ́AG[lƂ BUFFER_EMPTY, TIMEOUT,
     * PRECONDITION_NOT_MET, PORT_ERROR ԂB
     *
     * @return PORT_OK              I
     *         BUFFER_EMPTY         obt@͋ł
     *         TIMEOUT              ^CAEg
     *         PRECONDITION_NOT_MET O𖞂Ȃ
     *         PORT_ERROR           ̑̃G[
     *
     * @else
     *
     * @brief Reading data
     *
     * This function reads data from the buffer. If data is read
     * properly, this function will return PORT_OK return code. Except
     * normal return, BUFFER_EMPTY, TIMEOUT, PRECONDITION_NOT_MET and
     * PORT_ERROR will be returned as error codes.
     *  
     * @return PORT_OK              Normal return
     *         BUFFER_EMPTY         Buffer empty
     *         TIMEOUT              Timeout
     *         PRECONDITION_NOT_MET Preconditin not met
     *         PORT_ERROR           Other error
     *
     * @endif
     */
    virtual ReturnCode read(cdrMemoryStream& data);

    /*!
     * @if jp
     * @brief ڑ
     *
     * consumer, publisher, buffer ́E폜B
     *
     * @return PORT_OK
     *
     * @else
     *
     * @brief disconnect
     *
     * This operation destruct and delete the consumer, the publisher
     * and the buffer.
     *
     * @return PORT_OK
     *
     * @endif
     */
    virtual ReturnCode disconnect();

    /*!
     * @if jp
     * @brief ANeBu
     *
     * ̃RlN^ANeBu
     *
     * @else
     *
     * @brief Connector activation
     *
     * This operation activates this connector
     *
     * @endif
     */
    virtual void activate(){}; // do nothing

    /*!
     * @if jp
     * @brief ANeBu
     *
     * ̃RlN^ANeBu
     *
     * @else
     *
     * @brief Connector deactivation
     *
     * This operation deactivates this connector
     *
     * @endif
     */
    virtual void deactivate(){}; // do nothing

  protected:
    /*!
     * @if jp
     * @brief Buffer̐
     *
     * ^ꂽڑɊÂobt@𐶐B
     *
     * @param info ڑ
     * @return obt@ւ̃|C^
     *
     * @else
     * @brief create buffer
     *
     * This function creates a buffer based on given information.
     *
     * @param info Connector information
     * @return The poitner to the buffer
     *
     * @endif
     */
    virtual CdrBufferBase* createBuffer(ConnectorInfo& info);

    /*!
     * @if jp
     * @brief ڑmɃR[obNĂ
     * @else
     * @brief Invoke callback when connection is established
     * @endif
     */
    void onConnect();

    /*!
     * @if jp
     * @brief ڑؒfɃR[obNĂ
     * @else
     * @brief Invoke callback when connection is destroied
     * @endif
     */
    void onDisconnect();
    
    inline void onBufferRead(cdrMemoryStream& data)
    {
      m_listeners.
        connectorData_[ON_BUFFER_READ].notify(m_profile, data);

    }
    void onBufferEmpty(cdrMemoryStream& data)
    {
      m_listeners.
        connector_[ON_BUFFER_EMPTY].notify(m_profile);

    }
    void onBufferReadTimeout(cdrMemoryStream& data)
    {
      m_listeners.
        connector_[ON_BUFFER_READ_TIMEOUT].notify(m_profile);
    }

  private:
    /*!
     * @if jp
     * @brief InPortConsumer ւ̃|C^
     * @else
     * @brief the pointer to the InPortConsumer
     * @endif
     */
    InPortProvider* m_provider;

    /*!
     * @if jp
     * @brief ConnectorListenrs ւ̎Q
     * @else
     * @brief A reference to a ConnectorListener
     * @endif
     */
    ConnectorListeners& m_listeners;

    bool m_deleteBuffer;
  };
}; // namespace RTC

#endif  // RTC_PUSH_CONNECTOR_H
