// -*- C++ -*-
/*!
 * @file ListnerHolder.h
 * @brief Listener holder class
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2011
 *     Noriaki Ando
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */

#ifndef RTM_UTIL_LISTENERHOLDER_H
#define RTM_UTIL_LISTENERHOLDER_H

#include <vector>
#include <utility>
#include <coil/Mutex.h>
#include <coil/Guard.h>
#include <coil/NonCopyable.h>

namespace RTM
{
namespace util
{
  /*!
   * @if jp
   * @class Listener z_[NX
   *
   * ̃NX́AXiNX̒PȕێAǗsXiz_N
   * XłB̃NX𗘗p邽߂ɂ́Aev[g̑Pɓ
   * 郊XiNX (ListenerNX) сAListenerHolderNX
   * ev[gpāAۂɃXǐĂяos
   * ListenerHolderNXKvB
   *
   * ̃NX́AXbhZ[u邽߁AXi̒ǉƍ폜ɂ
   * Ă̓~[ebNXɂ郍bNsĂBSɃXbhZ[t
   * ȃXiǗ邽߂ɂ̓XĩR[obNR[ۂ
   * ~[ebNɂ郍bNsKvB
   *
   * @section ListenerNX̒`
   *
   * CxgɃR[obN郁o֐NX`
   * BR[obN̂߂̃o֐́ACӂ̖߂lA
   * ̂`łAʏ̊֐łĂ悢Aoperator()Ȃǂ̃t@N
   * ^ƂĒ`Ă悢Bۂɂ͊NXɂĂ̊֐
   * z֐ƂĒ`ÃNXpāAۂ̃XiNX
   * 邱ƂɂȂB܂AЂƂ̃XiNXɕɃR[obN֐
   * `Ă悢Bۂɂ́ÃR[obN֐ۂɌĂяo
   * @ɊւẮAListenerHolderNXɂďڂ`邱
   * ƂɂȂB
   * <pre>
   * class MyListenerBase
   * {
   * public:
   *   // R[obN֐1: ֐ĂяoZqɂR[obN֐
   *   // t@N^̂悤ɃR[obN֐`B
   *   virtual void operator()(std::string strarg) = 0; // z֐
   *   
   *   // R[obN̊֐VOj`lłꍇÂ悤ɒP
   *   // 郁o֐ƂĒ`邱Ƃ\B
   *   virtual void onEvent0(const char* arg0) = 0;
   *   virtual void onEvent1(int arg0) = 0;
   *   virtual void onEvent2(double arg0) = 0;
   *   virtual void onEvent3(HogeProfile& arg0) = 0;
   * };
   * </pre>
   *
   * @section ListenerHolderNX
   *
   * ListenerHolderNX͂LsitenerHolderNXev[gp
   * āAŒ` MyListenerBase NX̒ǉƍ폜ȂǊǗs
   * AۂɃR[obN֐Ăяo邱ƂɂȂB
   * ۂɃR[obNĂяoł́A֐VOj`푽lł
   * AЂƂ̃XiNX̃R[obN֐ꍇ
   * ߁Aʂ̃XiNXɑΉ邽߁ǍĂяoKvƂ
   * BListenerHolderNX́AMyListenerBaseNXƓVOj`
   * o֐A֐ł́AListenerHolderNXA
   * m_listeners, m_mutex ̂̃oϐ𗘗pāAo^
   * XiIuWFNg̃oϐĂяoB
   *
   * <pre>
   * class MyListenerHolderImpl
   *  : public ::RTM::util::ListenerHolder<MyListenerBase>
   * {
   * public:
   *   // ֐ĂяoZq̃R[obN֐̏ꍇ
   *   virtual void operator()(std::string strarg)
   *   {
   *     Gurad gurad(m_mutex);
   *     for (int i(0), len(m_listeners.size()); i < len; ++i)
   *     {
   *       m_listeners[i].first->operator()(strarg);
   *     }
   *   }
   *
   *   virtual void onEvent0(const char* arg0)
   *   {
   *     Gurad gurad(m_mutex);
   *     for (int i(0), len(m_listeners.size()); i < len; ++i)
   *     {
   *       m_listeners[i].first->onEvent(arg0);
   *     }
   *   }
   * };
   * </pre>
   *
   * XiIuWFNgւ̃|C^i[ĂEntryIuWFNg
   * std::pair<ListenerClass, bool> ƂĒ`ĂAfirst
   * ListenerIuWFNgւ̃|C^Asecond폜tOłB
   * āAXiIuWFNgփANZXꍇɂfirstgpB
   * }`Xbhŗp邱Ƃz肳ꍇ́AGuard
   * guard(m_mutex) ɂ郍bNYꂸɍsƁB
   * 
   * @section ListenerHolderNX̗p
   * ꂽMyListenerHolderImpl͈ƂĈȉ̂悤ɗpB
   *
   * <pre>
   * // Ƃ΃NXoƂĐ錾
   * MyListenerHolderImpl m_holder;
   *
   * // o^AN[[hœo^A
   * // IuWFNg̍폜HolderNXɔC
   * m_holder.addListener(new MyListener0(), true); // MyListener0
   * 
   * // R[obNĂяo
   * m_holder.operator()(strarg);
   * m_holder.onEvent0("HogeHoge);
   * </pre>
   *
   * @else
   *
   * @class Listener holder class
   *
   * @endif
   */
  template <typename ListenerClass>
  class ListenerHolder
    : public coil::NonCopyable
  {
  public:
    typedef coil::Guard<coil::Mutex> Guard;
    typedef std::pair<ListenerClass*, bool> Entry;
    typedef std::vector<Entry> EntryList;
    typedef typename EntryList::iterator EntryIterator;

    /*!
     * @if jp
     * @brief ListenerHolderNXRXgN^
     * @else
     * @brief ListenerHolder class ctor 
     * @endif
     */
    ListenerHolder()
    {
    }

    /*!
     * @if jp
     * @brief ListenerHolderfXgN^
     * @else
     * @brief ListenerHolder class dtor 
     * @endif
     */
    virtual ~ListenerHolder()
    {
      Guard guard(m_mutex);
      EntryIterator it(m_listeners.begin());
      
      for (; it != m_listeners.end(); ++it)
        {
          if ((*it).second)
            {
              delete (*it).first;
            }
          m_listeners.erase(it);
        }
    }
  
    /*!
     * @if jp
     * @brief Xiǉ
     * @else
     * @brief add listener object
     * @endif
     */
    virtual void addListener(ListenerClass* listener,
                     bool autoclean)
    {
      Guard guard(m_mutex);
      m_listeners.push_back(Entry(listener, autoclean));
    }
    
    /*!
     * @if jp
     * @brief Xi폜
     * @else
     * @brief remove listener object
     * @endif
     */
    virtual void removeListener(ListenerClass* listener)
    {
      Guard guard(m_mutex);
      EntryIterator it(m_listeners.begin());
      
      for (; it != m_listeners.end(); ++it)
        {
          if ((*it).first == listener)
            {
              if ((*it).second)
                {
                  delete (*it).first;
                }
              m_listeners.erase(it);
              return;
            }
        }
    }

  protected:
    /*!
     * @if jp
     * @brief Mutex
     * @else
     * @brief Mutex
     * @endif
     */
    coil::Mutex m_mutex;

    /*!
     * @if jp
     * @brief XĩXg
     * @else
     * @brief List of listener object
     * @endif
     */
    EntryList m_listeners;
  };
}; // util
}; // RTM

#define LISTENERHOLDER_CALLBACK(func, args)               \
  {                                                       \
    Guard guard(m_mutex);                                 \
    for (int i(0), len(m_listeners.size()); i < len; ++i) \
      {                                                   \
        m_listeners[i].first->func args;                  \
      }                                                   \
  }

#endif // RTM_UITL_LISTENERHOLDER_H
