// -*- C++ -*-
/*!
 * @file LocalServiceAdmin.h
 * @brief SDO service administration class
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2011
 *     Noriaki Ando
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: SdoConfiguration.cpp 1971 2010-06-03 08:46:40Z n-ando $
 *
 */

#ifndef RTC_LOCALSERVICEADMIN_H
#define RTC_LOCALSERVICEADMIN_H

#include <coil/Mutex.h>
#include <coil/Factory.h>
#include <coil/Singleton.h>

#include <rtm/idl/SDOPackageStub.h>
#include <rtm/SystemLogger.h>
#include <rtm/LocalServiceBase.h>

namespace RTM
{
  // forward decl
  class LocalServiceBase;
  typedef LocalServiceBase* (*ECNewFunc)();
  typedef void (*ECDeleteFunc)(LocalServiceBase* ls);
  typedef coil::Guard<coil::Mutex> Guard;

  /*!
   * @if jp
   *
   * @class LocalService administration class
   * @brief LocalService ǗNX
   *
   * TODO: Documentation
   *
   * @since 1.1.0
   *
   *
   * @else
   *
   * @class SDO service administration class
   * @brief SDO service administration class
   *
   * TODO: Documentation
   *
   * @since 1.1.0
   *
   * @endif
   */
  class LocalServiceAdmin
    : public coil::Singleton<LocalServiceAdmin>
  {
  public:
    /*!
     * @if jp
     *
     * @brief RXgN^
     * 
     * RXgN^
     *
     * @else
     *
     * @brief Constructor
     * 
     * Constructor
     *
     * @endif
     */
    LocalServiceAdmin();
    
    /*!
     * @if jp
     *
     * @brief zfXgN^
     * 
     * zfXgN^B
     * 
     * @else
     *
     * @brief Virtual destractor
     *
     * Virtual destractor.
     *
     * @endif
     */
    virtual ~LocalServiceAdmin();
    
    /*!
     * @if jp
     *
     * @brief LocaServiceAdmin̏
     *
     * TODO: Documentation
     *
     * @else
     *
     * @brief Initialization of LocalServiceAdmin
     *
     * TODO: Documentation
     *
     * @endif
     */
    void init(coil::Properties& props);
    
    /*!
     * @if jp
     *
     * @brief LocalserviceAdmin ̏I
     *
     * TODO: Documentation
     *
     * @else
     *
     * @brief Finalization ofLocalServiceAdmin
     *
     * TODO: Documentation
     *
     * @endif
     */
    void finalize();
    
    /*!
     * @if jp
     *
     * @brief LocalServiceProfileList̎擾
     * 
     * TODO: Documentation
     *
     * @else
     *
     * @brief Getting LocalServiceProfileList
     *
     * TODO: Documentation
     *
     * @endif
     */
    ::RTM::LocalServiceProfileList getServiceProfiles();
    
    /*!
     * @if jp
     *
     * @brief LocalServiceProfile 擾
     *
     * id Ŏw肳ꂽIDLocalService 
     * LocalServiceProfile 擾Bid  NULL |C^̏ꍇAw肳ꂽ
     * id ɊYServiceProfile ݂ȂꍇAfalseԂB
     *
     * @param id LocalService  IFR ID
     * @return w肳ꂽ id  LocalServiceProfile
     * 
     * @else
     *
     * @brief Get LocalServiceProfile of an LocalService
     *
     * This operation returns LocalServiceProfile of a LocalService
     * which has the specified id. If the specified id is
     * NULL pointer or the specified id does not exist in the
     * ServiceProfile list, false will be returned.
     *
     * @param id ID of an LocalService
     * @return LocalServiceProfile which has the specified id
     *
     * @endif
     */
    bool getServiceProfile(std::string name,
                           ::RTM::LocalServiceProfile& prof);
    
    /*!
     * @if jp
     *
     * @brief LocalService  Service 擾
     *
     * id Ŏw肳ꂽIDLocalService ̃|C^擾Bid 
     * NULL |C^̏ꍇAw肳ꂽ id ɊYServiceProfile 
     * ݂ȂꍇANULLԂB
     *
     * @param id LocalService  ID
     * @return w肳ꂽ id  LocalService ̃|C^
     * 
     * @else
     *
     * @brief Get a pointer of a LocalService
     *
     * This operation returnes a pointer to the LocalService
     * which has the specified id. If the specified id is
     * NULL pointer or the specified id does not exist in the
     * ServiceProfile list, NULL pointer will be returned.
     *
     * @param id ID of a LocalService
     * @return a pointer which has the specified id
     *
     * @endif
     */
    ::RTM::LocalServiceBase* getService(const char* id);
    
    /*!
     * @if jp
     * @brief SDO service provider Zbg
     *
     * TODO: Documentation
     * 
     * @else
     * @brief Set a SDO service provider
     *
     * TODO: Documentation
     *
     * @endif
     */
    bool addLocalService(::RTM::LocalServiceBase* service);
    
    /*!
     * @if jp
     * @brief LocalService 폜
     *
     * TODO: Documentation
     *
     * @else
     * @brief Remove a LocalService
     *
     * TODO: Documentation
     *
     * @endif
     */
    bool removeLocalService(const std::string name);
    
  private:
    /*!
     * @if jp
     * @brief w肳ꂽIDLǂ`FbN
     * @else
     * @brief Check if specified ID is enabled
     * @endif
     */
    bool isEnabled(const std::string& id, const coil::vstring& enabled);
    
    /*!
     * @if jp
     * @brief w肳ꂽIDłɑ݂邩ǂ`FbN
     * @else
     * @brief Check if specified ID is existing
     * @endif
     */
    bool notExisting(const std::string& id);
    
    /*!
     * @if jp
     * @brief Lock t SDO ServiceProfileList
     * @else
     * @brief SDO ServiceProfileList with mutex lock
     * @endif
     */
    std::vector<LocalServiceBase*> m_services;
    coil::Mutex m_services_mutex;
    
    /*!
     * @if jp
     * @brief logger
     * @else
     * @brief logger
     * @endif
     */
    ::RTC::Logger rtclog;
  };
}; // RTM

#endif // RTC_LOCALSERVICEADMIN_H
