// -*- C++ -*-
/*!
 * @file LocalServiceBase.h
 * @brief Local service provider base class and its factory
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2011
 *     Noriaki Ando
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */

#ifndef RTM_LOCALSERVICEBASE_H
#define RTM_LOCALSERVICEBASE_H

#include <vector>

#include <coil/Mutex.h>
#include <coil/Factory.h>
#include <coil/Timer.h>
#include <coil/Properties.h>

namespace RTM
{
  
  // forward decl
  class LocalServiceBase;
  
  /*!
   * @if jp
   * @brief LocalServicẽvt@Cf[^
   * @else
   * @brief Profile data structure of LocalService
   * @endif
   */
  class LocalServiceProfile
  {
  public:
    /*!
     * @if jp
     * @brief LocalServicẽT[rX
     * @else
     * @brief The name of LocalService
     * @endif
     */
    std::string name;
    /*!
     * @if jp
     * @brief LocalServicěŗLID
     * @else
     * @brief The unique ID of LocalService
     * @endif
     */
    std::string uuid;
    /*!
     * @if jp
     * @brief LocalServicẽvpeB
     * @else
     * @brief Properties of LocalService
     * @endif
     */
    coil::Properties properties;
    /*!
     * @if jp
     * @brief LocalServicẽ|C^
     * @else
     * @brief The pointer to LocalService
     * @endif
     */
    LocalServiceBase* service;
  };
  
  /*!
   * @if jp
   * @brief LocalServicẽXg
   * @else
   * @brief List of LocalServiceProfile
   * @endif
   */
  typedef std::vector<LocalServiceProfile> LocalServiceProfileList;
  
  /*!
   * @if jp
   *
   * @brief LocalService@NX
   *
   * Local Service Ƃ RT-Middlewarẽ}l[Wf[ŊeT[r
   * X񋟂邽߂̎dg݂łBLocal Serviceɂ͂̃[X
   * P[XlB
   *
   * 1. RTC̓WbNɑ΂ăT[rX񋟂BR|[lg̊J
   * ҂́A̒oāALocal ServicẽCX^Xւ̎QƂ
   * āÃT[rX𗘗p邱ƂłB
   *
   * 2. }l[Wɋ@\ǉ邽߂̃T[rXBƂ́ARTC̖O
   * o^CORBAl[T[rX̑ɁAVȃfBNgT[r
   * Xu[hLXgɂ閼Oʒm@\ȂǂVȋ@\}l[W
   * ɒǉ邽߂ɗp邱ƂłB}l[Wɂ́AlXȃANV
   * ɑ΂tbNpӂĂA𗘗p邱ƂŁA}l[W
   * ̗lXȃCxgɑ΂ăANVNƂłB
   *
   * 3. }l[W͎g̃T[rXȊOɕtIɊOɑ΂ăT[rX
   * 񋟂@\LBOCORBAoRŁA[J̋@\AƂ
   * ΃foCX⃊\[X̃}lWg@\ɑ΂ăANZX@
   * BT[rX̃CX^XɁA}l[Wɑ΂āA}l[W
   * T[rXƂēo^邱ƂŁAÕANZX󂯕t邱Ƃ
   * ł悤ɂȂB
   *
   * Local Service ̓W[Ƃă[hAʏB̃CX^X
   * BȂ킿ALocalService ̓VOgƂĎ
   * BCX^XꂽT[rX̓T[rXXgɓo^ARTC
   * ͖OUUIDɂăCX^X̎QƂ肷邱ƂłB
   *
   * ̃IuWFNg̃CtTCN͈ȉ̒ʂB
   *
   * -# IuWFNg͒ʏALIuWFNg (so, DLL) ƂăRpCE
   *    NB
   * -# }l[Wɑ΂ă[hƃW[֐ɂIu
   *    WFNgt@NgALocalServiceFactory ɑ΂ēo^B
   *    o^̃L[ɂUUIDƖOpAɂT[rXʂ
   *    B
   * -# rtc.conf̃RtBM[VwɂAL邱Ƃ
   *    w肳ĂT[rXCvoC_́ARTC̋NƓɃCX
   *    ^XB
   * -# CX^XA֐ init() Ă΂Bɂ͓YT[
   *    rX̂߂̃RtBM[VIvV coil::Property
   *    nB
   * -# }l[WT[rXIy[V reinit Ă΂ƁAׂĂ
   *     Local Service  reinit ĂяoAXVꂽRtBM[
   *     V񂪓ǂݍ܂B
   * -# }l[W̎ɂ́AׂĂ Local Service  finalizeĂ
   *       oÂ̂ŁAŃ\[X̉ȂǏIs
   *       B
   *
   * ̃NX̎ɓẮAȂƂȉ̏z֐
   * KvB
   *
   * - init(): ֐B^ꂽ RTObject  ServiceProfile 
   *   AYIuWFNgB
   * - reinit(): ď֐BServiceProfile ͐ݒXV̂ߓ
   *   IDŌĂяo邱ƂL邪A̍ۂɂ̊֐V
   *   ServiceProfile ƂƂɌĂяoB֐ł́Aݒ̕ύXȂ
   *   ďB
   * - getProfile(): ݒ肳ꂽvt@CԂ֐B
   * - finalize(): IBRV[}f^b`ۂɌĂяo
   *   ֐B֐ł͏IB
   *
   * LocalService ͒ʏ틤LIuWFNgƂăRpCEN
   * BLIuWFNg̃Gg|Cg͒ʏRpCꂽt@
   * C basename + "Init" ɂĂBȉɁANXAt@C
   * AGg|Cg֐̐B
   *
   * - NX: MyLocalService
   * - t@C: MyLocalService.h. MyLocalService.cpp
   * - LIuWFNg: MyLocalService.so (or DLL)
   * - Gg|Cg֐: MyLocalServiceInit()
   *
   * Gg|Cg֐͒ʏȉ̂悤ɁALocalServiceFactory
   * ɓYRV[}̃t@Ng (Ɖ̃t@N^) o^ȉ
   * 悤Ȋ֐ɂȂB
   *
   * <pre>
   * extern "C"
   * {
   *   void MyLocalServiceInit()
   *   {
   *     RTC::LocalServiceFactory& factory
   *       = RTC::LocalServiceFactory::instance();
   *     factory.addFactory(::RTC::MyLocalSerivce::name,
   *                        ::coil::Creator< ::RTC::LocalServiceBase,
   *                        ::RTC::MyLocalService>,
   *                        ::coil::Destructor< ::RTC::LocalServiceBase,
   *                        ::RTC::MyLocalService>);
   *   }
   * };
   * </pre>
   *
   * @else
   *
   * @endif
   *
   */
  class LocalServiceBase
  {
  public:
    /*!
     * @if jp
     * @brief zfXgN^
     * @else
     * @brief virtual destructor
     * @endif
     */
    virtual ~LocalServiceBase()
    {
    }
    
    /*!
     * @if jp
     * @brief LocalServiceNX̏֐
     *
     * ̃IuWFNg̏sBLocalService ꍇAO
     * ̐ݒ́Ainit()֐ɂ^B
     *
     * @param profile O^ꂽ LocalServiceProfile
     * @return ^ꂽ LocalServiceProfile s̏ꍇ false
     *
     * @else
     * @brief Initialization function of the LocalService class
     *
     * @endif
     */
    virtual bool
    init(const coil::Properties& props) = 0;
    /*!
     * @if jp
     * @brief LocalServiceNX̍ď֐
     *
     * TODO: hLg쐬
     *
     * @param profile Vɗ^ꂽ LocalServiceProfile
     * @return s LocalServiceProfile ^ꂽꍇ false
     *
     * @else
     * @brief Reinitialization function of the LocalService class
     *
     * TODO: Documentation
     *
     * @endif
     */
    virtual bool
    reinit(const coil::Properties& props) = 0;
    
    /*!
     * @if jp
     * @brief LocalServiceProfile Ԃ
     *
     * init()/reinit()ŗ^ꂽ LocalServiceProfile ͒ʏIuWFN
     * gŕێB̊֐ł͕ێĂ ServiceProfile 
     * B
     *
     * @return ̃IuWFNgێĂ LocalServiceProfile
     *
     * @else
     * @brief Getting LocalServiceProfile
     *
     * TODO: Documentation
     *
     * @endif
     */
    virtual const LocalServiceProfile&
    getProfile() const = 0;
    
    /*!
     * @if jp
     * @brief I
     *
     * LocalService IۂɌĂяoIp֐BYIu
     * WFNgێ郊\[XȂǂ̏sB
     *
     * @else
     * @brief Finalization
     *
     * TODO: Documentation
     *
     * @endif
     */
    virtual void
    finalize() = 0;
  };
  
  /*!
   * @if jp
   * @brief LocalServiceFactory  typedef
   * @else
   * @brief typedef of sdoServiceProviderFactory
   * @endif
   */
  typedef ::coil::GlobalFactory< ::RTM::LocalServiceBase > LocalServiceFactory;
  
#if defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__)
  /*!
   * @if jp
   * @brief NXev[g̖ICX^X
   * @else
   * @brief Explicit instantiation of class template
   * @endif
   */
  EXTERN template class DLL_PLUGIN
  ::coil::GlobalFactory< ::RTM::LocalServiceBase >;
#endif
}; // namespace RTM

#endif // RTM_LOCALSERVICEBASE_H
